/*
 * disp.c
 *
 * Created: 11.04.2017 11:22:42
 *  Author: dsosnowski
 */ 

#define F_CPU					8000000

//---includes-------------------
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include "disp.h"
#include "defs.h"

//------------------------------
static char digits[DIGIT_INDEX_LEN] = {~DIGIT_0, ~DIGIT_1, ~DIGIT_2, ~DIGIT_3, ~DIGIT_4, ~DIGIT_5, ~DIGIT_6, ~DIGIT_7, ~DIGIT_8, ~DIGIT_9, ~DIGIT_0, ~DIGIT_NULL, ~DIGIT_MINUS, ~DIGIT_DOT, ~DIGIT_POS};

struct {
	uint8_t d[DISP_NUMBEROFDISP];
	uint8_t emptysync;
} display;

//---------------------------
void DispInit(void){
	DISPLAY_PORT = 0xFF;
	DISPLAY_DDR = 0xFF;
	ANODE_PORT = (1<<ANODE1_PIN | 1<<ANODE2_PIN | 1<<ANODE3_PIN);
	ANODE_DDR = (1<<ANODE1_PIN | 1<<ANODE2_PIN | 1<<ANODE3_PIN);

	TCCR1A = (1<<WGM10 | 1<<WGM11);
	TCCR1B = (1<<WGM12 | 1<<WGM13);

	if (DISP_TIMER_DIV == 1) TCCR1B |= (1<<CS10);
	if (DISP_TIMER_DIV == 8) TCCR1B |= (1<<CS11);
	if (DISP_TIMER_DIV == 64) TCCR1B |= (1<<CS11 | 1<<CS10);
	if (DISP_TIMER_DIV == 256) TCCR1B |= (1<<CS12);
	if (DISP_TIMER_DIV == 1024) TCCR1B |= (1<<CS12 | 1<<CS10);
	
	OCR1A = (F_CPU / DISP_TIMER_DIV / DISP_REFRESH_FREQ);
	#if ((F_CPU / DISP_TIMER_DIV / DISP_REFRESH_FREQ) > 255) 
		#warning "sprawdz timer 1";
	#endif
	
	display.d[0] = digits[2];
	display.d[1] = digits[1];
	display.d[2] = digits[0];
	
	TIMSK = (1<<TOIE1);
	sei();
}

//-----------------------------------
ISR(TIMER1_OVF_vect)
{
	static uint8_t anode;
	
	anode++;
	if (anode >= DISP_CYCLES) anode = 0;
	
	switch (anode)
	{
		case 0:
			DISPLAY_PORT = display.d[0];
			ANODE1_SET;
			display.emptysync = 0;
		break;
		case 1:
			ANODE1_OFF;	
			DISPLAY_PORT = display.d[1];
			ANODE2_SET;
		break;
		case 2:
			ANODE2_OFF;
			DISPLAY_PORT = display.d[2];
			ANODE3_SET;
		break;
		case 3:
			ANODE3_OFF;
			DISPLAY_PORT = 0xFF;
			display.emptysync = 1;
		break;
	}
}

//---------------------------
void DispTest(uint8_t i){
	if (i < 8) {
		display.d[0] = (~(1<<i));
		display.d[1] = (~(1<<i));
		display.d[2] = (~(1<<i));	
	} else {
		display.d[0] = DIGIT_NULL_VAL;
		display.d[1] = DIGIT_NULL_VAL;
		display.d[2] = DIGIT_NULL_VAL;
	}
}

//---------------------------
void DispDigitsSet(uint8_t d2, uint8_t d1, uint8_t d0, uint8_t dot){
	if (d2 < DIGIT_INDEX_LEN) {
		display.d[2] = digits[d2];
		if (dot == 2) display.d[2] &= digits[DIGIT_DOT_INDEX];
	} else {
		display.d[2] = DIGIT_NULL_VAL;
	}
	if (d1 < DIGIT_INDEX_LEN) {
		display.d[1] = digits[d1];
		if (dot == 1) display.d[1] &= digits[DIGIT_DOT_INDEX];
	} else {
		display.d[1] = DIGIT_NULL_VAL;
	}
	if (d0 < DIGIT_INDEX_LEN) {
		display.d[0] = digits[d0];
		if (dot == 0) display.d[0] &= digits[DIGIT_DOT_INDEX];
	} else {
		display.d[0] = DIGIT_NULL_VAL;
	}	
}

//---------------------------
void DispDigitSet(uint8_t d, uint8_t index){
	if (d < DISP_NUMBEROFDISP){
		if (index < DIGIT_INDEX_LEN) {
			display.d[d] = digits[index];
		}
	}
}

//---------------------------
void DispDotSet(uint8_t d){
	if (d < DISP_NUMBEROFDISP){
		display.d[d] &= digits[DIGIT_DOT_INDEX];
	}
}

//---------------------------
void DispTemp(int16_t temp){
	uint8_t t[3];
	uint8_t dot;
	
	t[2] = DIGIT_NULL_INDEX;
	t[1] = DIGIT_NULL_INDEX;
	t[0] = DIGIT_NULL_INDEX;
	dot = 0xFF;
	
	if (temp > 1250) temp = 1250;
	if (temp < -550) temp = -550;
	
	if (temp >= 0){
		if (temp > 1000){
			t[2] = 1;
			temp -= 1000;		
			temp /= 10;
			t[1] = temp / 10;
			t[0] = temp % 10;
		} else {
			dot = 1;
			if (temp >= 100) {
				t[2] = temp / 100;
			}
			temp %= 100;
			t[1] = temp / 10;
			t[0] = temp % 10;			
		}
	} else {
		t[2] = DIGIT_MINUS_INDEX;
		temp *= -1;	

		if (temp >= 100) {
			temp /= 10;
		} else {
			dot = 1;
		}	
		t[1] = temp / 10;
		t[0] = temp % 10;
	}
	
	DispDigitsSet(t[2], t[1], t[0], dot);
}
